#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © Nekokatt 2019-2020
#
# This file is part of Hikari.
#
# Hikari is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hikari is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Hikari. If not, see <https://www.gnu.org/licenses/>.
"""Datastructure bases."""
__all__ = ["HikariEntity", "RawEntityT"]

import abc
import typing

import attr

from hikari.internal import marshaller

RawEntityT = typing.Union[
    None, bool, int, float, str, bytes, typing.Sequence[typing.Any], typing.Mapping[str, typing.Any]
]


@marshaller.marshallable()
@attr.s(slots=True, kw_only=True)
class HikariEntity(metaclass=abc.ABCMeta):
    """The base for any entity used in this API."""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © Nekoka.tt 2019-2020
#
# This file is part of Hikari.
#
# Hikari is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hikari is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Hikari. If not, see <https://www.gnu.org/licenses/>.
"""The logic for handling all requests to oauth2 endpoints."""

__all__ = ["RESTOauth2Component"]

import abc

from hikari.clients.rest_clients import component_base
from hikari import oauth2


class RESTOauth2Component(component_base.BaseRESTComponent, abc.ABC):  # pylint: disable=W0223
    """The REST client component for handling requests to oauth2 endpoints."""

    async def fetch_my_application_info(self) -> oauth2.Application:
        """Get the current application information.

        Returns
        -------
        :obj:`~hikari.oauth2.Application`
            An application info object.
        """
        payload = await self._session.get_current_application_info()
        return oauth2.Application.deserialize(payload)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © Nekoka.tt 2019-2020
#
# This file is part of Hikari.
#
# Hikari is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hikari is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Hikari. If not, see <https://www.gnu.org/licenses/>.
"""The abstract class that all REST client logic classes should inherit from."""

__all__ = ["BaseRESTComponent"]

import abc
import types
import typing

from hikari.internal import meta
from hikari.net import rest


class BaseRESTComponent(abc.ABC, metaclass=meta.UniqueFunctionMeta):
    """An abstract class that all REST client logic classes should inherit from.

    This defines the abstract method ``__init__`` which will assign an instance
    of :obj:`~hikari.net.rest.LowLevelRestfulClient` to the attribute that all
    components will expect to make calls to.
    """

    @abc.abstractmethod
    def __init__(self, session: rest.LowLevelRestfulClient) -> None:
        self._session: rest.LowLevelRestfulClient = session

    async def __aenter__(self) -> "BaseRESTComponent":
        return self

    async def __aexit__(
        self, exc_type: typing.Type[BaseException], exc_val: BaseException, exc_tb: types.TracebackType
    ) -> None:
        await self.close()

    async def close(self) -> None:
        """Shut down the REST client safely."""
        await self._session.close()

    @property
    def session(self) -> rest.LowLevelRestfulClient:
        """Get the :obj:`hikari.net.rest.LowLevelRestfulClient` session object."""
        return self._session
